from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.assay_result import AssayResult
from ..models.request_response_samples_item import RequestResponseSamplesItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestResponse")


@attr.s(auto_attribs=True)
class RequestResponse:
    """  """

    _results: Union[Unset, List[AssayResult]] = UNSET
    _samples: Union[Unset, List[RequestResponseSamplesItem]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        results: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._results, Unset):
            results = []
            for results_item_data in self._results:
                results_item = results_item_data.to_dict()

                results.append(results_item)

        samples: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._samples, Unset):
            samples = []
            for samples_item_data in self._samples:
                samples_item = samples_item_data.to_dict()

                samples.append(samples_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if results is not UNSET:
            field_dict["results"] = results
        if samples is not UNSET:
            field_dict["samples"] = samples

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        results = []
        _results = d.pop("results", UNSET)
        for results_item_data in _results or []:
            results_item = AssayResult.from_dict(results_item_data)

            results.append(results_item)

        samples = []
        _samples = d.pop("samples", UNSET)
        for samples_item_data in _samples or []:
            samples_item = RequestResponseSamplesItem.from_dict(samples_item_data)

            samples.append(samples_item)

        request_response = cls(
            results=results,
            samples=samples,
        )

        return request_response

    @property
    def results(self) -> List[AssayResult]:
        if isinstance(self._results, Unset):
            raise NotPresentError(self, "results")
        return self._results

    @results.setter
    def results(self, value: List[AssayResult]) -> None:
        self._results = value

    @results.deleter
    def results(self) -> None:
        self._results = UNSET

    @property
    def samples(self) -> List[RequestResponseSamplesItem]:
        if isinstance(self._samples, Unset):
            raise NotPresentError(self, "samples")
        return self._samples

    @samples.setter
    def samples(self, value: List[RequestResponseSamplesItem]) -> None:
        self._samples = value

    @samples.deleter
    def samples(self) -> None:
        self._samples = UNSET
